@@ -18,7 +18,6 @@ package org.apache.commons.betwixt;
 import java.util.ArrayList;
 import java.util.List;
 
-import org.apache.commons.betwixt.digester.XMLIntrospectorHelper;
 import org.apache.commons.betwixt.expression.Expression;
 
 /** <p><code>ElementDescriptor</code> describes the XML elements
@@ -79,6 +78,9 @@ public class ElementDescriptor extends NodeDescriptor {
 
     /** Whether this element refers to a primitive type (or property of a parent object) */
     private boolean primitiveType;
+    /** Is this a collective type? */
+    private boolean isCollectiveType;
+    
     /**
      * Is this element hollow?
      * In other words, is this descriptor a place holder indicating the name
@@ -98,6 +100,13 @@ public class ElementDescriptor extends NodeDescriptor {
       * or null if there is no separate implementation */
     private Class implementationClass = null;
     
+    /** 
+     * Should the bind time type determine the mapping? 
+     * (As opposed to the introspection time type.)
+     * Note that this attribute is write once, read many (WORM). 
+     */
+    private Boolean useBindTimeTypeForMapping = null;
+    
     /**  
      * Constructs an <code>ElementDescriptor</code> that refers to a primitive type.
      */
@@ -164,11 +173,13 @@ public class ElementDescriptor extends NodeDescriptor {
      } 
     
     /**
-     * Is this a simple element?
+     * <p>Is this a simple element?</p>
+     * <p>
      * A simple element is one without child elements or attributes.
      * This corresponds to the simple type concept used in XML Schema.
      * TODO: need to consider whether it's sufficient to calculate
-     * which are simple types (and so don't get IDs assigned etc)
+     * which are simple types (and so don't get IDs assigned etc).
+     * </p>
      * @return true if it is a <code>SimpleType</code> element
      */
     public boolean isSimple() {
@@ -297,15 +308,19 @@ public class ElementDescriptor extends NodeDescriptor {
       * passed in, unless some other matches the name exactly.
       *
       * @param name the localname to be matched, not null
-      * @returns the child ElementDescriptor with the given name if one exists, 
+      * @return the child ElementDescriptor with the given name if one exists, 
       * otherwise null
       */
     public ElementDescriptor getElementDescriptor(String name) {
     
         ElementDescriptor elementDescriptor = null;
         ElementDescriptor descriptorWithNullName = null;
+        ElementDescriptor firstPolymorphic = null;
         ElementDescriptor[] elementDescriptors = getElementDescriptors();
         for (int i=0, size=elementDescriptors.length; i<size; i++) {
+            if (firstPolymorphic == null && elementDescriptors[i].isPolymorphic()) {
+                firstPolymorphic = elementDescriptors[i];
+            }
             String elementName = elementDescriptors[i].getQualifiedName();
             if (name.equals(elementName)) {
                 elementDescriptor = elementDescriptors[i];
@@ -315,6 +330,9 @@ public class ElementDescriptor extends NodeDescriptor {
                 descriptorWithNullName = elementDescriptors[i];
             }
         }
+        if (elementDescriptor == null) {
+            elementDescriptor = firstPolymorphic;
+        }
         if (elementDescriptor == null) {
             elementDescriptor = descriptorWithNullName;
         }
@@ -544,22 +562,30 @@ public class ElementDescriptor extends NodeDescriptor {
     }
     
     /**
-     * TODO is this implementation correct?
-     * maybe this method is unnecessary
+     * Does this describe a collective?
      */
     public boolean isCollective() {
-        boolean result = false;
-        Class type = getPropertyType();
-        if (type != null) {
-            result = XMLIntrospectorHelper.isLoopType(type);
-        }
-        return result;
+        // TODO is this implementation correct?
+        // maybe this method is unnecessary
+        return isCollectiveType;
+    }
+    
+    /**
+     * Sets whether the element described is a collective.
+     * @since 0.7
+     * @param isCollectiveType
+     */
+    public void setCollective(boolean isCollectiveType) {
+        this.isCollectiveType = isCollectiveType;
     }
 
     /** 
-     * @todo is this really a good design?
+     * Finds the parent of the given descriptor.
+     * @param elementDescriptor <code>ElementDescriptor</code>
+     * @return <code>ElementDescriptor</code>, not null
      */
     public ElementDescriptor findParent(ElementDescriptor elementDescriptor) {
+        //TODO: is this really a good design?
         ElementDescriptor result = null;
         ElementDescriptor[] elementDescriptors = getElementDescriptors();
         for (int i=0, size=elementDescriptors.length; i<size; i++) {
@@ -591,13 +617,15 @@ public class ElementDescriptor extends NodeDescriptor {
     }
 
     /**
-     * Is this decriptor hollow?
+     * <p>Is this decriptor hollow?</p>
+     * <p>
      * A hollow descriptor is one which gives only the class that the subgraph
      * is mapped to rather than describing the entire subgraph.
      * A new <code>XMLBeanInfo</code> should be introspected 
      * and that used to describe the subgraph.
      * A hollow descriptor should not have any child descriptors. 
      * TODO: consider whether a subclass would be better
+     * </p>
      * @return true if this is hollow 
      */
     public boolean isHollow() {
@@ -612,9 +640,62 @@ public class ElementDescriptor extends NodeDescriptor {
      * and that used to describe the subgraph.
      * A hollow descriptor should not have any child descriptors. 
      * TODO: consider whether a subclass would be better
-     * @param true if this is hollow 
+     * @param isHollow true if this is hollow 
      */
     public void setHollow(boolean isHollow) {
         this.isHollow = isHollow;
     }  
+    
+    /**
+     * <p>Is the bind time type to be used to determine the mapping?</p>
+     * <p>
+     * The mapping for an object property value can either be the 
+     * introspection time type (based on the logical type of the property)
+     * or the bind time type (based on the type of the actual instance).
+     * </p>
+     * @since 0.7
+     * @return true if the bind time type is to be used to determine the mapping,
+     * false if the introspection time type is to be used
+     */
+    public boolean isUseBindTimeTypeForMapping() {
+        boolean result = true;
+        if ( this.useBindTimeTypeForMapping != null ) {
+            result = this.useBindTimeTypeForMapping.booleanValue();
+        }
+        return result;
+    }
+
+    /**
+     * <p>Sets whether the bind time type to be used to determine the mapping.
+     * The mapping for an object property value can either be the 
+     * introspection time type (based on the logical type of the property)
+     * or the bind time type (based on the type of the actual instance).
+     * </p><p>
+     * <strong>Note:</strong> this property is write once, read many.
+     * So, the first time that this method is called the value will be set
+     * but subsequent calls will be ignored.
+     * </p>
+     * @since 0.7
+     * @param useBindTimeTypeForMapping true if the bind time type is to be used to 
+     * determine the mapping, false if the introspection time type is to be used
+     */
+    public void setUseBindTimeTypeForMapping(boolean useBindTimeTypeForMapping) {
+        if ( this.useBindTimeTypeForMapping == null ) {
+            this.useBindTimeTypeForMapping = new Boolean(useBindTimeTypeForMapping);
+        }
+    }
+
+    /**
+     * <p>Is this a polymorphic element?</p>
+     * <p>
+     * A polymorphic element's name is not fixed at 
+     * introspection time and it's resolution is postponed to bind time.
+     * </p>
+     * @since 0.7
+     * @return true if {@link #getQualifiedName} is null, 
+     * false otherwise
+     */
+    public boolean isPolymorphic() { 
+        return (getQualifiedName() == null);
+    }
 }
