@@ -19,7 +19,6 @@ import java.beans.IntrospectionException;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collection;
-import java.util.HashMap;
 import java.util.Iterator;
 
 import org.apache.commons.betwixt.AttributeDescriptor;
@@ -28,7 +27,6 @@ import org.apache.commons.betwixt.Descriptor;
 import org.apache.commons.betwixt.ElementDescriptor;
 import org.apache.commons.betwixt.XMLBeanInfo;
 import org.apache.commons.betwixt.XMLIntrospector;
-import org.apache.commons.betwixt.digester.XMLIntrospectorHelper;
 import org.apache.commons.betwixt.expression.Context;
 import org.apache.commons.betwixt.expression.Expression;
 import org.apache.commons.betwixt.io.id.SequentialIDGenerator;
@@ -36,17 +34,10 @@ import org.apache.commons.collections.ArrayStack;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.xml.sax.Attributes;
+import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
 import org.xml.sax.helpers.AttributesImpl;
 
-// FIX ME!!!
-// Logging logic!
-
-// FIX ME!!
-// Error handling strategy!
-// i'm going to add SAXExceptions everywhere since it's the easiest way to make things work quick
-// but this is a poor strategy
-
 /**
   * <p>Abstract superclass for bean writers.
   * This class encapsulates the processing logic. 
@@ -92,8 +83,6 @@ public abstract class AbstractBeanWriter {
 
     /** Log used for logging (Doh!) */
     private Log log = LogFactory.getLog( AbstractBeanWriter.class );
-    /** Map containing ID attribute values for beans */
-    private HashMap idMap = new HashMap();
     /** Stack containing beans - used to detect cycles */
     private ArrayStack beanStack = new ArrayStack();
     /** Used to generate ID attribute values*/
@@ -186,6 +175,34 @@ public abstract class AbstractBeanWriter {
         end();
     }
     
+    /**
+     * <p>Writes the bean using the mapping specified in the <code>InputSource</code>.
+     * </p><p>
+     * <strong>Note:</strong> that the custom mapping will <em>not</em>
+     * be registered for later use. Please use {@link XMLIntrospector#register}
+     * to register the custom mapping for the class and then call
+     * {@link #write(Object)}.
+     * </p>
+     * @see #write(Object) since the standard notes also apply
+     * @since 0.7
+     * @param bean <code>Object</code> to be written as xml, not null
+     * @param source <code>InputSource/code> containing an xml document
+     * specifying the mapping to be used (in the usual way), not null
+     * @throws IOException
+     * @throws SAXException
+     * @throws IntrospectionException
+     */
+    public void write(Object bean, InputSource source) 
+    					throws IOException, SAXException, IntrospectionException {
+        writeBean(
+                	null, 
+                	null, 
+                	null, 
+                	bean, 
+                	makeContext( bean ), 
+                	getXMLIntrospector().introspect(bean.getClass(), source));
+    }
+    
     /** 
      * <p>Writes the given bean to the current stream 
      * using the given <code>qualifiedName</code>.</p>
@@ -197,6 +214,8 @@ public abstract class AbstractBeanWriter {
      * @param namespaceUri the namespace uri
      * @param localName the local name
      * @param qualifiedName the string naming root element
+     * @param introspectedBindType the <code>Class</code> of the bean 
+     * as resolved at introspection time, or null if the type has not been resolved
      * @param bean the <code>Object</code> to write out as xml
      * @param context not null
      * 
@@ -221,6 +240,83 @@ public abstract class AbstractBeanWriter {
         
         // introspect to obtain bean info
         XMLBeanInfo beanInfo = introspector.introspect( bean );
+        writeBean(namespaceUri, localName, qualifiedName, bean, context, beanInfo);
+        
+        log.trace( "Finished writing bean graph." );
+    }
+    
+    
+    private void writeBean (
+            String namespaceUri,
+            String localName,
+            String qualifiedName, 
+            Object bean,
+            ElementDescriptor parentDescriptor,
+            Context context) 
+                throws 
+                    IOException, 
+                    SAXException,
+                    IntrospectionException {                    
+    
+    if ( log.isTraceEnabled() ) {
+        log.trace( "Writing bean graph (qualified name '" + qualifiedName + "'" );
+    }
+    
+    // introspect to obtain bean info
+    XMLBeanInfo beanInfo = findXMLBeanInfo(bean, parentDescriptor);
+    writeBean(namespaceUri, localName, qualifiedName, bean, context, beanInfo);
+    
+    log.trace( "Finished writing bean graph." );
+}
+    
+    /**
+     * Finds the appropriate bean info for the given (hollow) element.
+     * @param bean
+     * @param parentDescriptor <code>ElementDescriptor</code>, not null
+     * @return <code>XMLBeanInfo</code>, not null
+     * @throws IntrospectionException
+     */
+    private XMLBeanInfo findXMLBeanInfo(Object bean, ElementDescriptor parentDescriptor) throws IntrospectionException {
+        XMLBeanInfo beanInfo = null;
+        Class introspectedBindType = parentDescriptor.getSingularPropertyType();
+        if ( introspectedBindType == null ) {
+            introspectedBindType = parentDescriptor.getPropertyType();
+        }
+        if ( parentDescriptor.isUseBindTimeTypeForMapping() || introspectedBindType == null ) {
+            beanInfo = introspector.introspect( bean );
+        } else {
+            beanInfo = introspector.introspect( introspectedBindType );
+        }
+        return beanInfo;
+    }
+
+    /**
+     * <p>Writes the given bean to the current stream 
+     * using the given mapping.</p>
+     *
+     * <p>This method will throw a <code>CyclicReferenceException</code> when a cycle
+     * is encountered in the graph <strong>only</strong> if the <code>getMapIDs()</code>
+     * setting of the <code>BindingConfiguration</code> is false.</p>
+     *
+     * @param namespaceUri the namespace uri, or null to use the automatic binding
+     * @param localName the local name  or null to use the automatic binding
+     * @param qualifiedName the <code>String</code> naming the root element 
+     *  or null to use the automatic binding
+     * @param bean <code>Object</code> to be written, not null
+     * @param context <code>Context</code>, not null
+     * @param beanInfo <code>XMLBeanInfo</code>, not null
+     * @throws IOException
+     * @throws SAXException
+     * @throws IntrospectionException
+     */
+    private void writeBean(
+            					String namespaceUri, 
+            					String localName, 
+            					String qualifiedName, 
+            					Object bean, 
+            					Context context, 
+            					XMLBeanInfo beanInfo) 
+    									throws IOException, SAXException, IntrospectionException {
         if ( beanInfo != null ) {
             ElementDescriptor elementDescriptor = beanInfo.getElementDescriptor();
             if ( elementDescriptor != null ) {
@@ -251,7 +347,7 @@ public abstract class AbstractBeanWriter {
                 } else {
                     pushBean ( context.getBean() );
                     if ( getBindingConfiguration().getMapIDs() ) {
-                        ref = (String) idMap.get( context.getBean() );
+                       ref = getBindingConfiguration().getIdMappingStrategy().getReferenceFor(context, context.getBean());
                     }
                     if ( ref == null ) {
                         // this is the first time that this bean has be written
@@ -259,7 +355,7 @@ public abstract class AbstractBeanWriter {
                         if (idAttribute == null) {
                             // use a generated id
                             id = idGenerator.nextId();
-                            idMap.put( bean, id );
+                            getBindingConfiguration().getIdMappingStrategy().setReference(context, bean, id);
                             
                             if ( getBindingConfiguration().getMapIDs() ) {
                                 // write element with id
@@ -272,6 +368,7 @@ public abstract class AbstractBeanWriter {
                                     beanInfo.getIDAttributeName(),
                                     id);
                                     
+
                             } else {    
                                 // write element without ID
                                 writeElement( 
@@ -285,8 +382,13 @@ public abstract class AbstractBeanWriter {
                         } else {
                             // use id from bean property
                             // it's up to the user to ensure uniqueness
-                            // XXX should we trap nulls?
-                            Object exp = idAttribute.getTextExpression().evaluate( context );
+                            Expression idExpression = idAttribute.getTextExpression();
+                            if(idExpression == null) {
+                                   throw new IntrospectionException(
+                                         "The specified id property wasn't found in the bean ("
+                                        + idAttribute + ").");
+                            }
+                            Object exp = idExpression.evaluate( context );
                             if (exp == null) {
                                 // we'll use a random id
                                 log.debug("Using random id");
@@ -296,7 +398,7 @@ public abstract class AbstractBeanWriter {
                                 // convert to string
                                 id = exp.toString();
                             }
-                            idMap.put( bean, id);
+                            getBindingConfiguration().getIdMappingStrategy().setReference(context, bean, id);
                             
                             // the ID attribute should be written automatically
                             writeElement( 
@@ -323,10 +425,8 @@ public abstract class AbstractBeanWriter {
                 }
             }
         }
-        
-        log.trace( "Finished writing bean graph." );
     }
-    
+
     /** 
       * Get <code>IDGenerator</code> implementation used to 
       * generate <code>ID</code> attribute values .
@@ -623,7 +723,8 @@ public abstract class AbstractBeanWriter {
             if ( log.isTraceEnabled() ) {
                 log.trace( "Element " + elementDescriptor + " is empty." );
             }
-        
+            context.pushOptions(elementDescriptor.getOptions());
+            
             Attributes attributes = addNamespaceDeclarations(
                 new ElementAttributes( elementDescriptor, context ), namespaceUri);
             writeContext.setCurrentDescriptor(elementDescriptor);
@@ -637,7 +738,7 @@ public abstract class AbstractBeanWriter {
             writeElementContent( elementDescriptor, context ) ;
             writeContext.setCurrentDescriptor(elementDescriptor);
             endElement( writeContext, namespaceUri, localName, qualifiedName );
-            
+            context.popOptions();
         }
     }
     
@@ -702,6 +803,7 @@ public abstract class AbstractBeanWriter {
                                     IntrospectionException {
                    
         if ( !ignoreElement( elementDescriptor, context ) ) {
+            context.pushOptions(elementDescriptor.getOptions());
             writeContext.setCurrentDescriptor(elementDescriptor);
             Attributes attributes = new IDElementAttributes( 
                         elementDescriptor, 
@@ -718,7 +820,7 @@ public abstract class AbstractBeanWriter {
             writeElementContent( elementDescriptor, context ) ;
             writeContext.setCurrentDescriptor(elementDescriptor);
             endElement( writeContext, namespaceUri, localName, qualifiedName );
-
+            context.popOptions();
         } else if ( log.isTraceEnabled() ) {
             log.trace( "Element " + qualifiedName + " is empty." );
         }
@@ -784,7 +886,7 @@ public abstract class AbstractBeanWriter {
                                 "",
                                 idrefAttributeName, 
                                 idrefAttributeName,
-                                "IDREF",
+                                "IDREF",    
                                 idrefAttributeValue);
         writeContext.setCurrentDescriptor(elementDescriptor);
         startElement( writeContext, uri, localName, qualifiedName, addNamespaceDeclarations(attributes, uri));        
@@ -836,6 +938,7 @@ public abstract class AbstractBeanWriter {
                                             localName, 
                                             qualifiedName, 
                                             object, 
+                                            childDescriptor,
                                             context );
                                 }
                             } else {
@@ -844,6 +947,7 @@ public abstract class AbstractBeanWriter {
                                             localName, 
                                             qualifiedName, 
                                             childBean, 
+                                            childDescriptor,
                                             context );
                             }
                         }                    
@@ -936,8 +1040,9 @@ public abstract class AbstractBeanWriter {
      * @param descriptor the <code>ElementDescriptor</code> to evaluate
      * @param context the <code>Context</code> against which the element will be evaluated
      * @return true if this element should be written out
+     * @throws IntrospectionException
      */
-    private boolean ignoreElement( ElementDescriptor descriptor, Context context ) {
+    private boolean ignoreElement( ElementDescriptor descriptor, Context context ) throws IntrospectionException {
         if ( ! getWriteEmptyElements() ) {
             return isEmptyElement( descriptor, context );
         }
@@ -951,16 +1056,19 @@ public abstract class AbstractBeanWriter {
      * and no body text.
      * For example, <code>&lt;element/&gt;</code> is an empty element but
      * <code>&lt;element attr='value'/&gt;</code> is not.</p>
-     *
+     * 
      * @param descriptor the <code>ElementDescriptor</code> to evaluate
      * @param context the <code>Context</code> against which the element will be evaluated
      * @return true if this element is empty on evaluation
+     * @throws IntrospectionException
      */
-    private boolean isEmptyElement( ElementDescriptor descriptor, Context context ) {
+    private boolean isEmptyElement( ElementDescriptor descriptor, Context context ) throws IntrospectionException {
+        //TODO: this design isn't too good
+        // to would be much better to render just once 
         if ( log.isTraceEnabled() ) {
             log.trace( "Is " + descriptor + " empty?" );
         }
-        
+                
         // an element which has attributes is not empty
         if ( descriptor.hasAttributes() ) {
             log.trace( "Element has attributes." );
@@ -979,7 +1087,7 @@ public abstract class AbstractBeanWriter {
         }
         
         // always write out loops - even when they have no elements
-        if ( XMLIntrospectorHelper.isLoopType( descriptor.getPropertyType() ) ) {
+        if ( descriptor.isCollective() ) {
             log.trace("Loop type so not empty.");
             return false;
         }
@@ -995,6 +1103,23 @@ public abstract class AbstractBeanWriter {
             }
         }
         
+        if ( descriptor.isHollow() )
+        {
+            Expression contentExpression = descriptor.getContextExpression();
+            if (contentExpression != null) {
+                Object childBean = contentExpression.evaluate(context);
+                if (childBean != null)
+                {
+                    XMLBeanInfo xmlBeanInfo = findXMLBeanInfo(childBean, descriptor);
+                    Object currentBean = context.getBean();
+                    context.setBean(childBean);
+                    boolean result = isEmptyElement(xmlBeanInfo.getElementDescriptor(), context);
+                    context.setBean(currentBean);
+                    return result;
+                }
+            }
+        }
+        
         log.trace( "Element is empty." );
         return true;
     }
@@ -1009,7 +1134,10 @@ public abstract class AbstractBeanWriter {
         private AttributeDescriptor[] attributes;
         /** Context to be evaluated when finding values */
         private Context context;
-
+        /** Cached attribute values */
+        private String[] values;
+        /** The number of unsuppressed attributes */
+        private int length;
         
         
         /** 
@@ -1019,8 +1147,36 @@ public abstract class AbstractBeanWriter {
          * @param context evaluate against this context
          */
         ElementAttributes( ElementDescriptor descriptor, Context context ) {
-            attributes = descriptor.getAttributeDescriptors();
             this.context = context;
+            init(descriptor.getAttributeDescriptors());
+        }
+        
+        private void init(AttributeDescriptor[] baseAttributes) {
+            attributes = new AttributeDescriptor[baseAttributes.length];
+            values = new String[baseAttributes.length];
+            int index = 0;
+            for (int i=0, size=baseAttributes.length; i<size; i++) {
+                AttributeDescriptor baseAttribute = baseAttributes[i];
+                String attributeValue = valueAttribute(baseAttribute);
+                if (attributeValue != null 
+                        && !context.getValueSuppressionStrategy()
+                        		.suppressAttribute(baseAttribute, attributeValue)) {
+                    values[index] = attributeValue;
+                    attributes[index] = baseAttribute;
+                    index++;
+                }
+            }
+            length = index;
+        }
+        
+        private String valueAttribute(AttributeDescriptor attribute) {
+            Expression expression = attribute.getTextExpression();
+            if ( expression != null ) {
+                Object value = expression.evaluate( context );
+                return convertToString( value, attribute, context );
+            }
+            
+            return "";
         }
         
         /**
@@ -1066,7 +1222,7 @@ public abstract class AbstractBeanWriter {
          * @return the number of attributes in this list
          */
         public int getLength() {
-            return attributes.length;
+            return length;
         }
         
         /** 
@@ -1153,14 +1309,8 @@ public abstract class AbstractBeanWriter {
          * @todo add value caching
          */
         public String getValue( int index ) {
-            if ( indexInRange( index )) {
-                Expression expression = attributes[index].getTextExpression();
-                if ( expression != null ) {
-                    Object value = expression.evaluate( context );
-                    return convertToString( value, attributes[index], context );
-                }
-                
-                return "";
+            if ( indexInRange( index ) ) {
+                return values[index];
             }
             return null;
         }
@@ -1197,7 +1347,7 @@ public abstract class AbstractBeanWriter {
          * @return true if the index with within the range of the attribute list
          */
         private boolean indexInRange( int index ) {
-            return ( index >= 0 && index < attributes.length );
+            return ( index >= 0 && index < getLength() );
         }
     }
     
@@ -1206,7 +1356,7 @@ public abstract class AbstractBeanWriter {
      * //TODO: refactor the ID/REF generation so that it's fixed at introspection
      * and the generators are placed into the Context.
      * @author <a href='http://jakarta.apache.org/'>Jakarta Commons Team</a>
-     * @version $Revision: 1.28 $
+     * @version $Revision: 201855 $
      */
     private class IDElementAttributes extends ElementAttributes {
 		/** ID attribute value */
@@ -1237,7 +1387,7 @@ public abstract class AbstractBeanWriter {
 			
 			// see if we have already have a matching attribute descriptor
 			AttributeDescriptor[] attributeDescriptors = descriptor.getAttributeDescriptors();
-			length = attributeDescriptors.length;
+			length = super.getLength();
 			for (int i=0; i<length; i++) {
 				if (idAttributeName.equals(attributeDescriptors[i])) {
 					matchingAttribute = true;
@@ -1697,7 +1847,7 @@ public abstract class AbstractBeanWriter {
     private String convertToString( Object value , Descriptor descriptor, Context context ) {
         return getBindingConfiguration()
             .getObjectStringConverter()
-                .objectToString( value, descriptor.getPropertyType(), null, context );
+                .objectToString( value, descriptor.getPropertyType(), context );
     }
     
     /**
