@@ -36,7 +36,7 @@ import org.apache.commons.logging.impl.SimpleLog;
   *
   * @author <a href="mailto:jstrachan@apache.org">James Strachan</a>
   * @author <a href="mailto:martin@mvdb.net">Martin van den Bemt</a>
-  * @version $Revision: 1.13 $
+  * @version $Revision: 164896 $
   */
 public class TestBeanWriter extends AbstractTestCase {
     
@@ -220,8 +220,7 @@ public class TestBeanWriter extends AbstractTestCase {
         String xml="<?xml version='1.0'?><beans>  <LoopBean name='Escape&lt;LessThan'/>"
             + "<LoopBean name='Escape&gt;GreaterThan'/><LoopBean name='Escape&amp;amphersand'/>"
             + "<LoopBean name='Escape&apos;apostrophe'/><LoopBean name='Escape&quot;Quote'/>"
-            + "<CustomerBean name='Escape&lt;LessThan' time='' date='' bigDecimal='' "
-            + "bigInteger='' timestamp='' ID='' nickName=''>"
+            + "<CustomerBean name='Escape&lt;LessThan' >"
             + "<projectMap/><projectNames/><emails><email>Escape&lt;LessThan</email>"
             + "<email>Escape&gt;GreaterThan</email><email>Escape&amp;amphersand</email>"
             + "<email>Escape'apostrophe</email><email>Escape\"Quote</email></emails>"
@@ -312,45 +311,7 @@ public class TestBeanWriter extends AbstractTestCase {
 //        baseLog.debug(out.getBuffer().toString());
         xmlAssertIsomorphicContent(parseString(out.getBuffer().toString()),parseString(xml), true);
     }
-    
-    /** Test nested case for writing empty elements */
-    public void testNestedWriteEmptyElements() throws Exception{
-        
-        // write same bean both times
-        LoopBean root = new LoopBean("base");
-        LoopBean middle = new LoopBean(null);
-        root.setFriend(middle);
-        middle.setFriend(new LoopBean(null));
-        
-//        SimpleLog baseLog = new SimpleLog( "[NestedEmpty]" );
-//        baseLog.setLevel(SimpleLog.LOG_LEVEL_TRACE);
-    
-        // test output when writing empty elements
-        StringWriter out = new StringWriter();
-        out.write("<?xml version='1.0'?>");
-        BeanWriter writer = new BeanWriter(out);
-        writer.setWriteEmptyElements(true);
-        writer.getBindingConfiguration().setMapIDs(false);
-        writer.write(root);
-//        baseLog.debug("NESTED EMPTY");
-//        baseLog.debug(out.getBuffer().toString());
-        String xml = "<?xml version='1.0'?><LoopBean><name>base</name><friend><name/><friend><name/></friend>"
-                    + "</friend></LoopBean>";
-        xmlAssertIsomorphicContent(parseString(out.getBuffer().toString()),parseString(xml), true);
-        
-        // test output when not writing empty elements
-        out = new StringWriter();
-        out.write("<?xml version='1.0'?>");
-        writer = new BeanWriter(out);
-        writer.setWriteEmptyElements(false);
-        writer.getBindingConfiguration().setMapIDs(false);
-        writer.write(root);
-//        baseLog.debug("NESTED NOT EMPTY");
-//        baseLog.debug(out.getBuffer().toString());
-        xml = "<?xml version='1.0'?><LoopBean><name>base</name></LoopBean>";
-        xmlAssertIsomorphicContent(parseString(out.getBuffer().toString()),parseString(xml), true);
-        
-    }
+
     
     public void testArrayWrite() throws Exception {
         ArrayBean bean = new ArrayBean("Rob");
